### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ###
# Author: Ben Weidmann
# Description: Analysis code for Team Players. Joint project with David Deming
# Contact: Ben Weidmann (benweidmann@g.havard.edu, email current as of May 2021)

# To replicate results:
  # change "ifp" to a local filepath (on line 23 of the code)
  # set gen_new_null = FALSE (see line 20; for fast replication; for full replication that recalculates null distributions for inference, set gen_new_null <- TRUE)

# Prelims -----------------------------------------------------------------
rm(list=ls())
library(readxl)
library(tidyverse)
library(estimatr)
library(arm)
library(mgcv)
library(stargazer)

# inference options
gen_new_null = FALSE # this adds computation time as it redoes the randomization inference

# specify file path to get saved data objects
ifp = "C:/Users/benwe/Dropbox/Uni/2016-17/Social Skills Project/EXPERIMENT/Teamplayer Data and Code/ECMA/Replication Materials"
  
# load experimental data
df_full = read_csv(paste0(ifp,"/D_data/verbose_data.csv")) # data at the individual level
df_all_expanded = read_csv(paste0(ifp,"/D_data/GroupTask_data.csv")) # data at the group level
allocation = read_csv(paste0(ifp,"/D_Data/allocation.csv")) # data that maps how individuals were allocated to groups

# read data for analysis of mechanisms (using the memory and shapes tasks). This analysis requires item-level data
dsha = read_csv(paste0(ifp,"/D_Data/group_shapes_data.csv")) # group SHAPES data for 'rushing' analysis [presented in section 5.2; note that each group appears to only have one participant, i.e. the reporter, who was responsible for entering answers] 
dmc = read_csv(paste0(ifp,"/D_Data/group_memory_timing.csv")) # group MEMORY timing data, for 'effort/motivation' analysis [presented in section 5.2; this data tracks how much time each individual spent memorizing different stimuli in the group memory task]
dmem_scores_task = read_csv(paste0(ifp,"/D_Data/group_memory_subtask_scores.csv")) # group MEMROY 
ifinal_mem = read_csv(paste0(ifp,"/D_Data/individual_memory_subtask_scores.csv")) # individual MEMORY data at the sub-task level [e.g. for 'stories/words/images']
group_times = read_csv(paste0(ifp,"/D_Data/group arrival time.csv")) # information about WHEN groupwork happened [i.e. admin data about time/date]
drep = read_csv(paste0(ifp,"/D_Data/reporter_status.csv")) # information about who the reporter is
dsurv = read_csv(paste0(ifp,"/D_Data/demographics.csv")) # information on age, gender and ethnicity

# change data type of "user" column (which is anonymized participant ID)
df_full$user = as.character(df_full$user) 
allocation$user = as.character(allocation$user)
dsha$user = as.character(dsha$user) 
dmc$user = as.character(dmc$user)
ifinal_mem$user = as.character(ifinal_mem$user)

# Generate useful summaries----
# Final participants: people we observed at least 3 times (in ANY group, not necessarily final groups)
final_participants = unique(df_full %>% filter(final_participant_flag==1) %>% pull(user))

# Final groups: groups with at least 2 final participants
final_groups = unique(df_full$group)

# individual skill summary (for people in the 343 final groups)
final_mem343_in = df_full %>% 
  group_by(user) %>% 
  summarize(opt_scaled_i=opt_scaled_i[1],
            shapes_scaled_i=shapes_scaled_i[1],
            memory_scaled_i=memory_scaled_i[1])

# Update allocation, to only include Final Groups (n=343)
allocation_final = allocation %>% filter(group %in% final_groups)

# Which round is each group in
group_round = df_all_expanded %>% group_by(group) %>% summarize(round=round[1])


# Table 1 -----------------------------------------------------------------
# Infomation on the final sample
# These stats apply to the 255 final participantswhere possible. We have n=252 for some categories.
round(median(dsurv$age,na.rm=T)) # median age
apply(dsurv[,c("female","latino_hispanic","black","asian","white")],2,mean,na.rm=T)

# Functions for section 4 analysis -----------------------------------------------
# Main analysis function
full_analysis = function(m1_form,df=df_all_expanded,
                          ci_level=0.95,
                          package="lmer", # defines the package we use to estimate models
                          inference="profile", # define inference method: options are 'profile'; 'normal'; 'randomization'
                          null_dist_name,
                          flex=F){ # allow gam?
  df$task = as.factor(df$task) # Preparation step: turn task into a factor
  
  # Fit model 1
  if(flex==FALSE) m1 = lm(as.formula(m1_form),df) # Step 1: fit model1
  if(flex==TRUE) m1 = mgcv::gam(formula = as.formula(m1_form),family=gaussian(),data=df) # Step 1: fit model1 [using spline]
  df$res = m1$residuals # get residuals
  Tg = df %>% group_by(group) %>% summarize(Tg = mean(res)) # Step 2: get averages for each group
  
  # In-between step: attach individual IDs to groups [and subset to remove people whose data we don't analyse]
  Tg_i = left_join(subset(allocation_final,group %in% Tg$group),Tg,by="group") # add individuals
  Tg_i = Tg_i %>% filter(user %in% final_participants) # Just include participants who will be analysed
    
  # Fit multilevel model
  if(package == "lmer"){
    mlm = lmer(Tg~1|user,Tg_i,REML=T)
    t_obs = sqrt(VarCorr(mlm)$user[1])
  }
  if(package == "nlme"){
    require(nlme)
    mlm = lme(Tg~1,data=Tg_i,random=~1|user,method="ML") # fit model 4
    t_obs = as.numeric(VarCorr(mlm)["(Intercept)","StdDev"])
  }
  
  # tidy
  beta_fixef = Tg_i %>% group_by(user) %>% 
    summarize(beta_fixef = mean(Tg)) %>% 
    arrange(beta_fixef) 

# Print inference info about team player effect
  if(inference=="normal"){
    cire = round(intervals(mlm,level=ci_level)$reStruct$user,3)
    rownames(cire) = "SD of beta, using RE:"
    print(cire)}
  if(inference=="profile"){
    print(paste0("team player estimate = ",round(sqrt(VarCorr(mlm)$user[1]),3)))
    print(confint(mlm,parm=".sig01",level=ci_level))  
  }
  if(inference=="randomization"){
    print(paste0("Doing randomization inference for ",null_dist_name))
    null_dist = get(null_dist_name)
    p_val = mean(null_dist>t_obs)
    print(paste("RI p-val = ",round(p_val,4)))
    print(paste0("t_obs = ",round(t_obs,3)))
    null_dist = unlist(null_dist)
    null_plot = qplot(null_dist,bins=30)
    print(null_plot+geom_vline(xintercept = t_obs)+theme_bw())
  }

return(list(betas=beta_fixef, 
            m1=m1, # model 1
            Tg=Tg, # 'Group efficiency' i.e. how well groups turn individual skills into group performance
            mlm=mlm, # multilevel model 
            t_obs = t_obs
))
}


# Randomization Inference functions -----------------------------------
draw_null = function(m1_form,nsims,package="lmer",flex_null=F,df_inf=df_all_expanded){
  # This function draws under the null that "only individual skill matters"
  # flex_null is whether we want to be able to specify a gam for model 1

  # First, fit the model we'll use and generate the residuals
  # Work with a temporary data.frame called "df_inf"
  if(flex_null==FALSE) mod_inf = lm(as.formula(m1_form),df_inf)
  if(flex_null==TRUE) mod_inf = mgcv::gam(as.formula(m1_form),family=gaussian(),data=df_inf)
  df_inf$res = mod_inf$resid
  
  # Set up table to house simulation
  sigma_null = rep(NA,nsims)
  
  # Draw null distribution
  for(j in 1:nsims){
    
    # Generate table with new allocation
      allocation_round = left_join(allocation,group_round,by="group")
      null_allocation_temp = allocation_round %>% 
        group_by(round) %>% mutate(temp.id = sample(user)) %>% 
        dplyr::select(-user)

    # Make sure this looks like allocation final (i.e. has same 343 groups)
    null_allocation_temp = null_allocation_temp %>% 
      filter(group %in% final_groups) %>% filter(!is.na(temp.id))
    
    # triple this, so we have a row for each group-task-user, rather than just group-task
    null_allocation_temp3 = rbind(null_allocation_temp,null_allocation_temp,null_allocation_temp)
    null_allocation_temp3$task = rep(c("opt","memory","shapes"),each=nrow(null_allocation_temp))
    
    # match in Tg [from the'res' column]
    null_allocation_temp3$group = as.character(null_allocation_temp3$group)
    null_allocation_temp3 = left_join(null_allocation_temp3,df_inf[,c("group","task","res")],by=c("group","task"))
    null_allocation_temp3 = null_allocation_temp3 %>% filter(task %in% df_inf$task) # this allows us to subset df by task
    
    # Calculate Tg_i [i.e. get the data to a group/user level]
    Tg_i.temp = null_allocation_temp3 %>% group_by(group,temp.id) %>% 
      summarize(Tg=mean(res)) %>% rename(user=temp.id) %>% filter(user %in% final_participants)
    
    # Fit our multilevel model
    if(package=="nlme"){
      mtemp = lme(Tg~1,data=Tg_i.temp,random=~1|user,method="ML") # fit model 4
      sigma_null[j] = as.numeric(VarCorr(mtemp)[1,"StdDev"])}
    if(package=="lmer"){
      mtemp = suppressWarnings(suppressMessages(lmer(Tg~(1|user),Tg_i.temp))) # Suppression here is because we have a bunch of variance=0, which we get warned about.
      sigma_null[j] = as.numeric(sqrt(VarCorr(mtemp)$user[1]))
    }
}
  return(sigma_null)
}

# Save null distribution (using a particular model, defined by m1_form)
gen_save_null = function(cov_summary=cov_temp,
                         m1_form=m1_temp,
                         nsims=5000,
                         flex=F,
                         df = df_all_expanded){ 
  title = paste("null",cov_summary,sep="_") # name of null distribution
  set.seed(1)
  assign(title,draw_null(m1_form,nsims,flex_null=flex,df_inf = df)) # generate null (using draw_null) and assign it to the "title"
  null_out = get(title)
  write.csv(null_out,paste0(ifp,"/D_Data/Nulls/",title,".csv"),row.names = F) # save it
  return(null_out)
}



# SECTION 4 ---------------------------------------------------------------
# Section 4.1: Are some people team players? ------------------------------
# Table 2: Summarizing the teamplayer effect ------------------------------

# Model using normal approximation for CI
# Profile likelihood
  # Define the primary model
    # Note, in response to reviewer feedback, we removed the indicator variables D_know and D_prev from the pre-specified model. This has no effect
  pre_registered_mod = "scaled_score~Xscore*task-Xscore-task"
  t2_profile = full_analysis(pre_registered_mod,
                              package="lmer",inference="profile",
                              ci_level=0.932)
  summary(t2_profile$m1)

# Randomization inference
  # define the parameters of our null analysis
  cov_temp = "preregistered" # name of model1
  m1_temp = pre_registered_mod # definition of model1
  title_temp = paste("null",cov_temp,sep="_") # summary name of null distribution
  if(gen_new_null==T) assign(title_temp,gen_save_null()) # if we want to generate a new null, do it here
  if(gen_new_null==F) assign(title_temp,read.csv(paste0(ifp,"/D_Data/Nulls/",title_temp,".csv"))) # otherwise, grab old null

  # Generate RI p-value
  t2_RI = full_analysis(m1_form = pre_registered_mod, 
                        package="lmer",
                        inference = "randomization",
                        null_dist = title_temp)
  summary(t2_RI$m1)

  # This is just to show that our normal CI has a p-value of <0.001
  t2_normal = full_analysis(pre_registered_mod,
                            package="nlme",
                            inference="normal",ci_level = 0.999)
  
# Save beta data ----------------------------------------------------------
# Table where each row is a participant/group/session combination
  beta_out = df_full %>% group_by(group,sessionID,user) %>% 
    summarize(eyes_sum_i=eyes_sum_i[1],
              agreeaeble_i=agreeable_i[1],
              conscientious_i=conscientious_i[1],
              extravert_i=extravert_i[1]) %>%
    filter(user %in% final_participants)
  beta_out = left_join(beta_out,t2_profile$betas,"user") %>% 
    rename(beta_i = beta_fixef) 
  
# Rescale, so that beta_i and beta_g are standardized
  scalex = function(vec) (vec-mean(vec,na.rm=T))/sd(vec,na.rm=T)
  beta_out$beta_i = scalex(beta_out$beta_i)
  beta_out_g = beta_out %>% group_by(group) %>% summarize(beta_max_g = max(beta_i),
                                                          beta_min_g = min(beta_i),
                                                          beta_mean_g = mean(beta_i),
                                                          n_g = n())
  # join in group data
  beta_out = left_join(beta_out,beta_out_g,"group")  

# Save
  beta_out = left_join(beta_out,group_times,"group")
  beta_out = beta_out %>% arrange(group) %>% 
    dplyr::select(group,user,lab_session,month,day,time_began,beta_i,beta_max_g,beta_min_g,n_g,everything())
  # if(savestuff==TRUE) write.csv(x = beta_out,file=paste0(ifp,"/Data/beta_out.csv"),row.names=F)
    

# Table 2 continued: Team players (with different model 1 controls) ----------------------
# Table 2, Column 2: no controls
  cov_temp = "nocovs" # name of model1
  m1_temp = "scaled_score~1" # definition of model1
  title_temp = paste("null",cov_temp,sep="_") # summary name of null distribution
  if(gen_new_null==T) assign(title_temp,gen_save_null()) # if we want to generate a new null, do it here
  if(gen_new_null==F) assign(title_temp,read.csv(paste0(ifp,"/D_Data/Nulls/",title_temp,".csv"))) # otherwise, grab old null
  
  # Generate p-value from RI
  t2_nocovs = full_analysis(m1_form = m1_temp, 
                            package="lmer",
                            inference = "randomization",
                            null_dist = title_temp)
  
  # Generate p-value from profile likelihood
  t2_nocovs = full_analysis(m1_form = m1_temp, 
                            package="lmer",
                            inference = "profile",
                            null_dist = title_temp,ci_level = 0.998)
  "No need to calculate p-val for normal approx; this has to be less than 0.001"

# Table 2, Column 3: ravens only
  cov_temp = "ravens" # name of model 1
  m1_temp = "scaled_score~ravens_sum" # definition of model1
  title_temp = paste("null",cov_temp,sep="_") # summary name of null distribution
  if(gen_new_null==T) assign(title_temp,gen_save_null()) # if we want to generate a new null, do it here
  if(gen_new_null==F) assign(title_temp,read.csv(paste0(ifp,"/D_Data/Nulls/",title_temp,".csv"))) # otherwise, grab old null
  
  # Generate p-value for RI
  t2_ravens_RI = full_analysis(m1_form = m1_temp, 
                                 package="lmer",
                                 inference = "randomization",
                                 null_dist = title_temp)
  summary(t2_ravens_RI$m1)

  # Generate p-value for profile likelihood
  t2_ravens_profile = full_analysis(m1_form = m1_temp, 
                          package="lmer",
                          inference = "profile",
                          null_dist = title_temp,ci_level = 0.998)
  summary(t2_ravens_profile$m1)
  
  # p-value for normal approx
  "Again, no need, as for col 1 it was p<0.001, so it has to be here, too"
  
# Table A2 (robustness of Table 2): Flexible functions --------------------------------------------
  # Col 1
  "Already have this"

  # Col 2: Xmax only
  cov_temp = "Xmax_only" # name of model1
  m1_temp = "scaled_score~Xmax*task-Xmax-task" # definition of model1
  title_temp = paste("null",cov_temp,sep="_") # summary name of null distribution
  if(gen_new_null==T) assign(title_temp,gen_save_null()) # if we want to generate a new null, do it here
  if(gen_new_null==F) assign(title_temp,read.csv(paste0(ifp,"/D_Data/Nulls/",title_temp,".csv"))) # otherwise, grab old null
  
  # Generate p-value
  App_t4_Xmax = full_analysis(m1_form = m1_temp, 
                                 package="lmer",
                                 inference = "randomization",
                                 null_dist = title_temp)
  summary(App_t4_Xmax$m1)

  
# col 3: Xmin only
  cov_temp = "Xmin_only" # name of model1
  m1_temp = "scaled_score~Xmin*task-Xmin-task+know+prev" # definition of model1
  title_temp = paste("null",cov_temp,sep="_") # summary name of null distribution
  if(gen_new_null==T) assign(title_temp,gen_save_null()) # if we want to generate a new null, do it here
  if(gen_new_null==F) assign(title_temp,read.csv(paste0(ifp,"/D_Data/Nulls/",title_temp,".csv"))) # otherwise, grab old null
  
  # Generate p-value
  App_t4_Xmin = full_analysis(m1_form = m1_temp, 
                                 package="lmer",
                                 inference = "randomization",
                                 null_dist = title_temp)
  summary(App_t4_Xmin$m1)
  
# col 4: X1, X2, X3
  cov_temp = "X1_X2_X3" # name of model1
  m1_temp = "scaled_score~Xmax*task+Xmed*task+Xmin*task-Xmax-Xmed-Xmin-task" # definition of model1
  title_temp = paste("null",cov_temp,sep="_") # summary name of null distribution
  if(gen_new_null==T) assign(title_temp,gen_save_null()) # if we want to generate a new null, do it here
  if(gen_new_null==F) assign(title_temp,read.csv(paste0(ifp,"/D_Data/Nulls/",title_temp,".csv"))) # otherwise, grab old null
  
  # Generate p-value
  App_t4_X1_X2_X3 = full_analysis(m1_form = m1_temp, 
                              package="lmer",
                              inference = "randomization",
                              null_dist = title_temp)
  summary(App_t4_X1_X2_X3$m1)

# col 5: FLEXIBLE MODELS
  # for this analysis, we need to create three new variables, because gams don't estimate interactions
  df_all_expanded$Xscore_mem = ifelse(df_all_expanded$task=="memory",df_all_expanded$Xscore,0)  
  df_all_expanded$Xscore_sha = ifelse(df_all_expanded$task=="shapes",df_all_expanded$Xscore,0)
  df_all_expanded$Xscore_opt = ifelse(df_all_expanded$task=="opt",df_all_expanded$Xscore,0)
  
  cov_temp = "flex_Xsum" # name of model1
  m1_temp = "scaled_score~s(Xscore_mem)+s(Xscore_sha)+s(Xscore_opt)" # definition of model1
  title_temp = paste("null",cov_temp,sep="_") # summary name of null distribution
  if(gen_new_null==T) assign(title_temp,gen_save_null(flex=T)) # if we want to generate a new null, do it here
  if(gen_new_null==F) assign(title_temp,read.csv(paste0(ifp,"/D_Data/Nulls/",title_temp,".csv"))) # otherwise, grab old null
  
  # Generate p-value
  App_t4_flex_Xsum = full_analysis(m1_form = m1_temp,
                                  package="lmer",
                                  inference = "randomization",
                                  flex=T,
                                  null_dist = title_temp)
  summary(App_t4_flex_Xsum$m1)
  plot(App_t4_flex_Xsum$m1)
  summary(App_t4_flex_Xsum$mlm)  
  
# col 6: Ravens_Max
  cov_temp = "ravens_max" # name of model1
  m1_temp = "scaled_score~ravens_max" # definition of model1
  title_temp = paste("null",cov_temp,sep="_") # summary name of null distribution
  if(gen_new_null==T) assign(title_temp,gen_save_null()) # if we want to generate a new null, do it here
  if(gen_new_null==F) assign(title_temp,read.csv(paste0(ifp,"/D_Data/Nulls/",title_temp,".csv"))) # otherwise, grab old null
  
  # Generate p-value
  App_t4_ravens_max = full_analysis(m1_form = m1_temp,
                                   package="lmer",
                                   inference = "randomization",
                                   null_dist = title_temp)
  summary(App_t4_ravens_max$m1)
  summary(App_t4_ravens_max$mlm)  
  
# col 7: Ravens_Min
  cov_temp = "ravens_min" # name of model1
  m1_temp = "scaled_score~ravens_min" # definition of model1
  title_temp = paste("null",cov_temp,sep="_") # summary name of null distribution
  if(gen_new_null==T) assign(title_temp,gen_save_null()) # if we want to generate a new null, do it here
  if(gen_new_null==F) assign(title_temp,read.csv(paste0(ifp,"/D_Data/Nulls/",title_temp,".csv"))) # otherwise, grab old null
  
  # Generate p-value
  App_t4_ravens_min = full_analysis(m1_form = m1_temp,
                                    package="lmer",
                                    inference = "randomization",
                                    null_dist = title_temp)
  summary(App_t4_ravens_min$m1)
  summary(App_t4_ravens_min$mlm)  
  
# col 8: RMET_max
  cov_temp = "RMET_max" # name of model1
  m1_temp = "scaled_score~RMETmax" # definition of model1
  title_temp = paste("null",cov_temp,sep="_") # summary name of null distribution
  if(gen_new_null==T) assign(title_temp,gen_save_null()) # if we want to generate a new null, do it here
  if(gen_new_null==F) assign(title_temp,read.csv(paste0(ifp,"/D_Data/Nulls/",title_temp,".csv"))) # otherwise, grab old null
  
  # Generate p-value
  App_t4_RMETmax = full_analysis(m1_form = m1_temp,
                                    package="lmer",
                                    inference = "randomization",
                                    null_dist = title_temp)
  summary(App_t4_RMETmax$m1)
  summary(App_t4_RMETmax$mlm)  
  
  
# col 9: RMET_min
  cov_temp = "RMET_min" # name of model1
  m1_temp = "scaled_score~RMETmin" # definition of model1
  title_temp = paste("null",cov_temp,sep="_") # summary name of null distribution
  if(gen_new_null==T) assign(title_temp,gen_save_null()) # if we want to generate a new null, do it here
  if(gen_new_null==F) assign(title_temp,read.csv(paste0(ifp,"/D_Data/Nulls/",title_temp,".csv"))) # otherwise, grab old null
  
  # Generate p-value
  App_t4_RMETmin = full_analysis(m1_form = m1_temp,
                                 package="lmer",
                                 inference = "randomization",
                                 null_dist = title_temp)
  summary(App_t4_RMETmin$m1)
  summary(App_t4_RMETmin$mlm)  

# col 10: all
  cov_temp = "all" # name of model1
  m1_temp = "scaled_score~Xmax*task+Xmed*task+Xmin*task-Xmax-Xmed-Xmin-task+ravens_min+ravens_max+RMETmin+RMETmax" # definition of model1
  title_temp = paste("null",cov_temp,sep="_") # summary name of null distribution
  if(gen_new_null==T) assign(title_temp,gen_save_null()) # if we want to generate a new null, do it here
  if(gen_new_null==F) assign(title_temp,read.csv(paste0(ifp,"/D_Data/Nulls/",title_temp,".csv"))) # otherwise, grab old null
  
  # Generate p-value
  App_t4_all = full_analysis(m1_form = m1_temp,
                                 package="lmer",
                                 inference = "randomization",
                                 null_dist = title_temp)
  summary(App_t4_all$m1)
  summary(App_t4_all$mlm)  

    
# Table 3: Validation using Cryptography ---------------------------
# Here, we examine how predictive variables are of the crypto scores
# First, gather information on teamplayer endowment from main model (table1, col1)
  betas = t2_profile$betas

  #  Calculate group-level endowments
  beta_group = left_join(allocation_final,
                         betas %>% mutate(beta_1sig = (beta_fixef>=sd(beta_fixef))),
                         "user") %>% group_by(group) %>% 
      dplyr::summarize(beta_max = max(beta_fixef,na.rm = T),
                       beta_min = min(beta_fixef,na.rm=T),
                       beta_mean = mean(beta_fixef,na.rm=T),
                       beta_1sig = max(beta_1sig,na.rm=T))
  
  # Add group beta information onto cryptography scores
  dcry_scoresMOD = df_full %>% group_by(group) %>% 
    dplyr::select(group,cryp_scoreFINAL) %>%
    filter(!is.na(cryp_scoreFINAL)) %>%
    summarize(cryp_scoreFINAL=cryp_scoreFINAL[1]) %>% 
    left_join(.,beta_group,"group")

  # Add ravens_mean and rmet_mean (group-level endowment variables)
  group_endomwnents_in = df_all_expanded %>% group_by(group) %>% 
    summarize(ravens_mean = ravens_sum[1]/3,
              RMET_mean = RMETsum[1]/3)
  dcry_scoresMOD = left_join(dcry_scoresMOD,group_endomwnents_in,"group")
  
  # Scale everything [except the group id]
  dcry_scoresMOD[,-1] = dcry_scoresMOD[,-1] %>% mutate_all(scalex)

  # Regressions for Table 3 [in original submission this was table 4, hence the t4 label]
  t4col1 = lm(cryp_scoreFINAL~beta_mean,dcry_scoresMOD)
  t4col2 = lm(cryp_scoreFINAL~beta_max,dcry_scoresMOD)
  t4col3 = lm(cryp_scoreFINAL~beta_min,dcry_scoresMOD)
  t4col4 = lm(cryp_scoreFINAL~beta_1sig,dcry_scoresMOD)
  t4col5 = lm(cryp_scoreFINAL~ravens_mean,dcry_scoresMOD)
  t4col6 = lm(cryp_scoreFINAL~RMET_mean,dcry_scoresMOD)
  
  # Save
  stargazer(t4col1,t4col2,t4col3,t4col4,t4col5,t4col6,
            out=paste0(ifp,"/E_regression output/TABLE 3_Cryptography and Beta.html"),
            column.sep.width = "-15pt",
            dep.var.labels = "Crypto Score (scaled)",
            report=c("vc*p*s"))

# Table 4: the team player index and demographics -------------------------
  # Generate user-level demographic data
  demo_corr = df_full %>% 
    group_by(user) %>%
    summarize(post_secondary_years = post_secondary_years_i[1],
              under_rep_minority = under_rep_minority_i[1],
              female = female_i[1],
              age = age_i[1],
              extravert = extravert_i[1],
              conscientious = conscientious_i[1],
              agreeable = agreeable_i[1]) %>%
    filter(user %in% final_participants)
  
  # establish a table with beta values
  dbeta = t2_profile$betas
  
  # add demographic and personality variables
  dbeta = left_join(dbeta,demo_corr,"user")
  
  # standardize
  dbeta = dbeta %>% mutate_at(.vars=vars(post_secondary_years,age,female,beta_fixef,agreeable,extravert,conscientious),scalex)
  
  # models
  mdem1 = lm(beta_fixef~female,dbeta) # female
  mdem2 = lm(beta_fixef~post_secondary_years,dbeta) # post_secondary_years
  mdem3 = lm(beta_fixef~age,dbeta) # age
  mdem4 = lm(beta_fixef~under_rep_minority,dbeta) # under represented minority (african american; native american; latino)
  mdem5 = lm(beta_fixef~agreeable+conscientious+extravert,dbeta)
  
  # save
  stargazer(out=paste0(ifp,"/E_regression output/TABLE 4_Demographics and Beta.html"),
            mdem1,mdem2,mdem3,mdem4,mdem5,
            column.sep.width = "-30pts",
            report=c("vc*p*s"))
  
# Figure 3 (correlates of beta) -----------------------------------
  other_in = df_full %>% 
    group_by(user) %>% 
    summarize(RMET = eyes_scaled_i[1],
              shapes = shapes_scaled_i[1])
  dbeta = left_join(dbeta,other_in,"user")
  dbeta$post_secondary_years2 = dbeta$post_secondary_years^2  
  
  # Standardize everything
  scaled_vars = names(dbeta)[-1]
  dbeta = dbeta %>% mutate_at(.vars=vars(scaled_vars),scalex)  
  
# Function to generate plot
plot_beta = function(xvar,col,xlabel,df=dbeta,save=FALSE,size=12){
  df = as.data.frame(df)
  df$x = scalex(df[,xvar])
  pb = ggplot(data=df,aes(x = x,y=beta_fixef))+
    geom_point(col=col,alpha=0.2,size=2)+
    theme_bw()+ggtitle("")+theme(text = element_text(size=16))+
    xlab(xlabel)+ylab("")+
    geom_smooth(method="lm",col=col)
  print(pb)
  if(save==TRUE) ggsave(pb,file=paste0(ifp,"Charts/plots/beta vs ",xvar,".png"),height=size,width=size,units="cm")
  return(pb)
}


# Figure 2 ----------------------------------------------------------------
# Beta and Ravens (Figure 2, left panel)
pb_ravens = plot_beta(xvar="shapes",col="red",xlabel="Ravens Score [Scaled]",save=FALSE)

# Beta and RMET (Figure 2, right panel)
pb_rmet = plot_beta(xvar="RMET",col="blue",xlabel="RMET Score [Scaled]",save=FALSE)

# calculate the correlations for the tables below Figures 2
  # Simple correlations [column 1]
    cor.test(dbeta$beta_fixef,dbeta$shapes) # Ravens and pre-registered beta
    cor.test(dbeta$beta_fixef,dbeta$RMET) # RMET and pre-registered beta

  # Correlations with beta from NULL model [column 2]
    beta_nocovs = t2_nocovs$betas %>% rename(beta_nocovs = beta_fixef)
    dbeta_extra = left_join(dbeta,beta_nocovs,"user")
    cor.test(dbeta_extra$beta_nocovs,dbeta_extra$shapes) # Ravens and nocovs beta
    cor.test(dbeta_extra$beta_nocovs,dbeta_extra$RMET) # RMET and nocovs beta


# Figure A9 ---------------------------------------------------------------
  # Beta and BIG5
    pb_agreeable = plot_beta(xvar="agreeable",col="dark green",xlabel="Agreeableness [Scaled]",save=FALSE,size=11)
    pb_conscientious = plot_beta(xvar="conscientious",col="dark green",xlabel="Conscientiousness [Scaled]",save=FALSE,size=11)
    pb_extravert = plot_beta(xvar="extravert",col="dark green",xlabel="Extraversion [Scaled]",save=FALSE,size=11)
    
    # calculate the correlations for the plot
    cor.test(dbeta$beta_fixef,dbeta$agreeable) # Agreeableness and pre-registered beta
    cor.test(dbeta$beta_fixef,dbeta$conscientious) # Conscientiousness and pre-registered beta
    cor.test(dbeta$beta_fixef,dbeta$extravert) # Extraversion and pre-registered beta
    
    # correlation between no_covs beta and big5
    cor.test(dbeta_extra$beta_nocovs,dbeta_extra$agreeable) # Agreeable and nocovs beta
    cor.test(dbeta_extra$beta_nocovs,dbeta_extra$conscientious) # Conscientiousness and nocovs beta
    cor.test(dbeta_extra$beta_nocovs,dbeta_extra$extravert) # Extraversion and nocovs beta
    
    
# Section 4.2 expanded: who are team players ------------------------------
# RMET and gender ---------------------------------------------------------
# How does RMET differ by gender for the final participants?
# Add raw RMET score into dbeta
dbeta = df_full %>% group_by(user) %>% 
      summarize(eyes_sum = eyes_sum_i[1]) %>%
      filter(user %in% final_participants) %>% 
      left_join(.,dbeta,"user")

# Difference by gender [for FN in section 4.2]----
t.test(dbeta$eyes_sum~dbeta$female)
"Men get an average of 18; women 18.7; pvalue = 0.111"

# Caption for Figure 2----
mean(dbeta$eyes_sum)
"mean RMET score is 18.5"

# Age and team player index -----------------------------------------------
# Correlation between age and beta: in the text we report a null result
cor.test(dbeta_extra$age,dbeta_extra$beta_fixef)

# Ethnicity and the team player index -------------------------------------
# Correlation between ethnicity and beta: in the text we report a null result
cor.test(dbeta_extra$under_rep_minority,dbeta_extra$beta_fixef)

### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ###
# SECTION 5 ---------------------------------------------------------------
# APPENDIX Table A8: beta and RMET (relationship persists) -------
t6col1 = lm(beta_fixef~RMET,dbeta)  # just RMET
t6col2 = lm(beta_fixef~RMET+age+female+post_secondary_years+post_secondary_years2,dbeta) # add demographics
t6col3 = lm(beta_fixef~RMET+age+female+post_secondary_years+post_secondary_years2+shapes+agreeable+extravert+conscientious,dbeta) # everything!
t6col4 = lm(beta_fixef~age+female+post_secondary_years+post_secondary_years2+shapes+agreeable+extravert+conscientious,dbeta) # everything BUT RMET

stargazer(out=paste0(ifp,"/E_regression output/TABLE A8_beta hat and RMET.html"),
          t6col1,t6col2,t6col3,t6col4,
          column.sep.width = "-15pts",star.cutoffs = c(0.05,0.01),
          report=c("vc*s"))

# APPENDIX Table A9: beta_nocovs and interactions -------------------------
  dbeta_extra$beta_nocovs = scalex(dbeta_extra$beta_nocovs)
  App_t6col1 = lm(beta_nocovs~RMET,dbeta_extra)  # just RMET
  App_t6col2 = lm(beta_nocovs~RMET*shapes,dbeta_extra)  # RMET ravens interaction
  App_t6col3 = lm(beta_nocovs~RMET*shapes+age+female+post_secondary_years+post_secondary_years2+agreeable+extravert+conscientious,dbeta_extra)  # ravens*RMET + all
  App_t6col4 = lm(beta_nocovs~RMET*female,dbeta_extra)  # female*RMET interaction
  App_t6col5 = lm(beta_nocovs~RMET*female+age+shapes+post_secondary_years+post_secondary_years2+agreeable+extravert+conscientious,dbeta_extra)  # female*RMET + all
  
  stargazer(out=paste0(ifp,"/E_regression output/TABLE A9_beta hat no covs vs variables and interactions.html"),
            App_t6col1,App_t6col2,App_t6col3,App_t6col4,App_t6col5,
            column.sep.width = "-15pts",star.cutoffs = c(0.05,0.01),
            report=c("vc*s"))
              
# Add combined measure of social skill (S) ---------------------
  # We do a very simple thing, and just take the average of betas and RMET
  dbeta$Smean = 1/2*(scalex(dbeta$beta_fixef)+scalex(dbeta$RMET))
  
  # add to "allocation" so we can get group summaries
  allocationSmean = left_join(allocation_final,dbeta[,c("Smean","user")],"user")
  S_groups = allocationSmean %>% 
    group_by(group) %>%
    summarize(Smean_mean=mean(Smean,na.rm=T),
              Smean_max=max(Smean,na.rm=T))
  
# Table 5: what predicts cryptography (S and others) ----------------------
  # add the Smean information to cryptography scores
  dcry_scoresMOD = left_join(dcry_scoresMOD,S_groups,"group")
  
  # Grab some other group measures that we compare to cryptography scores
  # We will scale these shortly
  temp_in = df_all_expanded %>% 
    group_by(group) %>% 
    summarize(ravens_max=ravens_max[1],
              extravert_mean=extravert_sum[1],
              agreeable_mean=agreeable_sum[1],
              conscientious_mean=conscientious_sum[1])
  dcry_scoresMOD = left_join(dcry_scoresMOD,temp_in,"group")
  
  # Rename and rescale
  scale_vars = names(dcry_scoresMOD)[-1]
  dcry_scoresMOD = dcry_scoresMOD %>% mutate_at(.vars=vars(scale_vars),scalex)
  
  # Fit models [note that in the original submission, this was table 7, hence the labels]
  mod7col1 = lm(cryp_scoreFINAL~RMET_mean,dcry_scoresMOD)
  mod7col2 = lm(cryp_scoreFINAL~Smean_mean,dcry_scoresMOD)
  mod7col3 = lm(cryp_scoreFINAL~ravens_mean,dcry_scoresMOD)
  mod7col4 = lm(cryp_scoreFINAL~Smean_mean+ravens_mean,dcry_scoresMOD)

  # Save
  stargazer(out=paste0(ifp,"/E_regression output/TABLE 5_Cryptography and Social Skills.html"),
            mod7col1,mod7col2,mod7col3,mod7col4,
            colum.sep.width = "-15pts",star.cutoffs = c(0.05,0.01),
            report=c("vc*p*s"))

# Rushing the shapes task -------------------------------------------------
  # How many items did groups answer?
  # First, need to adapt data from subtest1 [where we recorded "------" as 'no answer']
  dsha_num = dsha %>% filter(response!="------")
  
  # Now, we can get the 'number' of responses by counting the rows for each group
  dsha_num = dsha %>% group_by(subtest,group) %>% summarize(nattempted = n()) # how many items did groups answer?
  
  # add percent correct
  dsha_time = dsha %>% group_by(group) %>% summarize(correct = sum(correct),subtest=subtest[1])
  dsha_time$n = NA # number of items in the form
  dsha_time$n[dsha_time$subtest=="subtest1"] = 13
  dsha_time$n[dsha_time$subtest=="subtest2"] = 14
  dsha_time$n[dsha_time$subtest=="subtest3"] = 13
  dsha_time$n[dsha_time$subtest=="subtest4"] = 10
  dsha_time$percent_correct = dsha_time$correct/dsha_time$n
  dsha_time = left_join(dsha_time,dsha_num,by=c("group","subtest")) # add n_attempted
  
  # what was the time limit?
  dsha_time$lim = 180 
  dsha_time$lim[dsha_time$subtest=="subtest2"] = 240
  
  # Take item-level data (dsha) and summarize tasktime and subtest info
  dshat = dsha %>% group_by(group) %>% summarize(time=tasktime[1],subtest=subtest[1])
  dsha_time = left_join(dsha_time,dshat[,c("group","time")],"group") # add this to percent_correct, and num_attempted
  dsha_time$n_correct = dsha_time$n*dsha_time$percent_correct # how many did groups get correct
  
  # which groups rushed?
  # We define rushing as: not using all the time, despite not answering all the questions
  dsha_time$answer_all = ifelse(dsha_time$nattempted==dsha_time$n,1,0)
  
  # Did people use less than the full time? [i.e. hit submit before the time finished?]
  dsha_time$notlim = ifelse(dsha_time$time<dsha_time$lim,1,0)
  paste0("number of groups who used full time = ",sum(dsha_time$notlim==0))
  paste0("percent of groups who used full time = ",round(sum(dsha_time$notlim==0)/nrow(dsha_time),3))
  
  # what percentage of groups "rushed"
  rushing_cutoff = 15
  dsha_time$rushing = ifelse(dsha_time$time+rushing_cutoff<dsha_time$lim,1,0)
  paste0("number of groups who RUSHED = ",sum(dsha_time$rushing))
  paste0("percent of groups who used RUSHED = ",round(sum(dsha_time$rushing)/nrow(dsha_time),3))
  
  # Did rushing make you do worse?
  t.test(dsha_time$percent_correct~dsha_time$rushing)
  
  # Add in whether or not teamplayers were present
  beta_in = data.frame(user=betas$user,beta=betas$beta_fixef,stringsAsFactors = F)
  beta_in = beta_in %>% mutate(beta = scalex(beta),
                               Bplayer1sig = as.numeric(beta>sd(beta)),
                               BplayerHalfsig = as.numeric(beta>0.5*sd(beta)))
  allocation_beta = left_join(allocation_final,beta_in,"user")
  beta_group = allocation_beta %>% filter(group %in% final_groups)
  beta_group = beta_group %>% 
    group_by(group) %>%
    summarize(beta_mean=mean(beta,na.rm=T),
              Bplayerin1sig = max(Bplayer1sig,na.rm=T))
  
  # Add beta information onto dsha_time [shapes data, with timing info]
  dsha_time = left_join(dsha_time,beta_group,"group")
  
  # Is rushing associated with having a team player?
  t.test(rushing~Bplayerin1sig,dsha_time)

  # BPlayer_mean
  cor.test(dsha_time$rushing,dsha_time$beta_mean)

  # Conscientiousness
  temp_in = df_all_expanded %>% 
    group_by(group) %>% 
    summarize(RMET=RMETsum[1],conscientious=conscientious_sum[1]) %>% 
    left_join(.,S_groups[,c("group","Smean_mean")],"group")
  dsha_time = left_join(dsha_time,temp_in,"group") 
  cor.test(dsha_time$rushing,dsha_time$RMET)
  cor.test(dsha_time$rushing,dsha_time$Smean_mean)
  
# Mechanisms: Memory Analysis ---------------------------------------------
# 5.3: Allocative efficiency ----------------------------------------------
  # Unfortunately, for coding/flow reasons, this comes out of order in terms of the paper (i.e. before the rest of section 5.2)
  
  # How many stimuli did participants look at?
  ntype = dmc %>% group_by(group,block,user) %>% summarize(n_type = length(unique(domain)))
  ntype_group = ntype %>% group_by(group) %>% summarize(max_type = max(n_type))
  paste0("% of groups who went for the 1-person-per-stimulus strategy = ",round(mean(ntype_group$max_type==1),4))

  # Add in group memory scores
  dmem_score_final = df_all_expanded %>% filter(task=="memory") %>%
    dplyr::select(group,scaled_score)
  ntype = left_join(ntype,dmem_score_final,by="group") %>% 
    rename(memory_scaled=scaled_score)
  
  # Just include groups who went for the "1-person-per-stimulus" strategy
  singles = ntype_group %>% filter(max_type==1) %>% pull(group)
  dmc1 = dmc %>% filter(group %in% singles)
  
# How well did groups sub-task differentiate? -----------------------------
  # Get individual memory scores and allocation
  ifinal_mem[,-1] = apply(ifinal_mem[,-1],2,scalex) # scale everything
  ifinal_mem = ifinal_mem %>% rename(stories=story) # rename cols
  
  # Get optimum allocation for each group
    # Function to generate memory ranks  
    mem_pred = function(df,actual){
      memdf = as.matrix(df[,c("images","stories","words")])
      poss = permn(1:3)
      mem_score = rep(NA,length(poss))
      for(i in 1:length(poss)) mem_score[i] = sum(c(memdf[1,poss[[i]]][1],
                                                    memdf[2,poss[[i]]][2],
                                                    memdf[3,poss[[i]]][3]))
      # save
      out = data.frame(max=NA,rank=NA)
      out$max=max(mem_score) # get max
      out$rank = match(actual,sort(mem_score,decreasing=T))
      return(out)
    }
  
  # For each group:
  Gm = data.frame(group=unique(dmc1$group),best_pred=NA,actual_pred=NA,rank=NA,stringsAsFactors = F)
    # Predicted sum of skill based on actual strategy (actual_pred)
    # Predicted sum of skill based on optimum strategy (best_pred)
  require(combinat)
  for(g in 1:nrow(Gm)){
    ids = allocation %>% filter(group==Gm$group[g]) # who was in each group
    imem = ifinal_mem %>% filter(user %in% ids$user) %>% arrange(user) # what were individual memory scores
    act = dmc1 %>% filter(group==Gm$group[g]) %>% arrange(user) %>% pull(domain) # what did people look at?
    act_pred = rep(NA,3) # what would we predict as the outcome? [set up vector to capture individual strength]
    for(i in 1:3) act_pred[i] = imem[i,act[i]]
    apt = sum(unlist(act_pred))
    Gm$actual_pred[g] = apt
    best = mem_pred(imem,apt)
    Gm$best_pred[g] = best$max
    Gm$rank[g] = best$rank
  }
  
  # Generate strat score
  Gm$strat_score = Gm$best_pred-Gm$actual_pred
  
  # Does strategy score correlate with doing well?
  Gmout = left_join(Gm,dmem_score_final,"group")
  
  # Did teams with a good strategy out-perform expectations?
  Gmout = Gmout %>% mutate_at(.vars=vars(strat_score,rank,scaled_score),scalex)
  cor.test(Gmout$rank,Gmout$scaled_score) # lower rank means better strategy
  cor.test(Gmout$strat_score,Gmout$scaled_score) # lower strat score means better strategy
  "Correlations between strategy and memory performance is weak and positive"
  
  # Does 'rank' correlate with Social Skill estimates?
  # Join in Splayer data
  GmBeta = left_join(Gmout,S_groups,"group")
  cor.test(GmBeta$Smean_mean,-GmBeta$strat_score)
  cor.test(GmBeta$Smean_mean,-GmBeta$rank)
  "Mean social skill does NOT correlate with either measure of strategy"
  

# 5.2 Motivation (Memory) -------------------------------------------------
  # Prepare information on each group's subtask score [subtask domain is captured in "domain"]
  dmem_scores_task = dmem_scores_task %>% rename(domain=task2)
  dmem_scores_task$domain[dmem_scores_task$domain=="story"] = "stories"
  
  # Join into our dmc1 data
  dmc2 = left_join(dmc1,dmem_scores_task,by=c("group","domain"))
  
  # add on individual memory score, for the relevant domain
  imem = pivot_longer(data = ifinal_mem,
                      cols = c("images","stories","words"),
                      names_to = "domain",values_to = "iscore")
  dmc2 = left_join(dmc2,imem,c("user","domain"))
  mmem = lm(ave~iscore,dmc2) # "ave" is the score of group g, on memory domain d
  dmc2$resid = resid(mmem)
  
  ### ### ### ### ### ### ##  
  # Analysis with RMET
  drmet = df_full %>% group_by(user) %>% 
    summarize(rmet=eyes_scaled_i[1])
  drmet$rmet = scalex(drmet$rmet) # scale wrt to final participants
  drmet$Rplayer1sig = ifelse(drmet$rmet>1,1,0) # Identify "RMET players" [people>1sd above average]
  dmc2 = left_join(dmc2,drmet,by="user")
  dmc2 = dmc2 %>% group_by(group) %>% mutate(Rplayerin1sig = max(Rplayer1sig),
                                             Rsum = sum(rmet),
                                             n_others = n()-1)
  
  # Analyse the effect of being in a "high RMET" environment
  dmc2 = dmc2 %>% ungroup() %>% mutate(Rbar_gi=(Rsum-rmet)/(n_others)) # Rbar_gi is the mean RMET in group g, NOT counting individual i
  
  # Scale predictors
  dmc2$Rbar_gi = scalex(dmc2$Rbar_gi)
  dmc2$iscore = scalex(dmc2$iscore)

# Table 6: Motivation and memory sub-task analysis ------------------------
  require(estimatr)
  dmc2$mem_standard = scalex(dmc2$ave)
  
  # Fit models (note that we have a few non final_participants in final_groups)
  # Note: in original submission this was table8, hence the labels
  t8_col1 = lm_robust(mem_standard~iscore+Rbar_gi+block+domain,dmc2,clusters = group)
  summary(lm(mem_standard~iscore+Rbar_gi+block+domain,dmc2)) # compare to non-robust SE
  summary(t8_col1)
  round(summary(t8_col1)$coef,3)
  
  # add RMET mean
  t8_col2 = lm_robust(mem_standard~iscore+Rbar_gi+rmet+block+domain,dmc2,clusters = group)
  summary(lm(mem_standard~iscore+Rbar_gi+rmet+block+domain,dmc2))
  round(summary(t8_col2)$coef,3)

# Appendices ----------------------------------------------------------------
# Appendix Table A1: Robustness of table 2 to Editor's suggested approaches
  # There were two suggestions: 
    # Examine the impact of removing/adding  familiarity variables (col2)
    # Estimate in a single step (col3)
  
  # Col 1: we have from above
  # Col 2: just add group familiarity
  cov_temp = "preregistered_familiarity" # name of model1
  m1_temp = "scaled_score~Xscore*task-Xscore-task+know+prev" # definition of model1
  title_temp = paste("null",cov_temp,sep="_") # summary name of null distribution
  if(gen_new_null==T) assign(title_temp,gen_save_null()) # if we want to generate a new null, do it here
  if(gen_new_null==F) assign(title_temp,read.csv(paste0(ifp,"/D_data/Nulls/",title_temp,".csv"))) # otherwise, grab old null
  
  # Generate RI p-value
  tA0_RI = full_analysis(m1_form = m1_temp, 
                        package="lmer",
                        inference = "randomization",
                        null_dist = title_temp)
  summary(tA0_RI$m1)
  
  # Get value from profile likelihood
  tA0_RI = full_analysis(m1_form = m1_temp, 
                         package="lmer",
                         inference = "profile",
                         ci_level = 0.925,
                         null_dist = title_temp)
  summary(tA0_RI$m1)

# Editor's suggested alternative analysis (simultaneous analysis)----  
  # First, need formula to generate null hypothesis
  draw_null_ecma = function(m1_form,nsims,package="lmer",flex_null=F,df_inf=df_all_expanded){
    # This function draws under the null that "only individual skill matters"
    # it is ADAPTED from draw_null, so that we can use ecma's suggested approach (i.e. individual fixed effects in model 1)
    # flex_null is whether we want to be able to specify a gam for model 1
    
    # The idea here is that we:
    # condition group scores with actual \sum_Xi of original group
    # but, in model 1, create a new imatrix [i.e. fixed effects]
    # this new imatrix randomly jumbles people within rounds
    # note that this function doesn't engage with session fixed effects
    
    # jumble allocation matrix, within round
    allocation_round = left_join(allocation,group_round,by="group")
    null_allocation_temp = allocation_round %>% 
      group_by(round) %>% mutate(temp.id = sample(user))
    
    # put this "temp.id" into df_full [which we'll use in model2b]
    df_full_temp = df_full %>% arrange(group,user)
    df_full_temp = left_join(df_full_temp,null_allocation_temp,c("group","user"))
    df_full_temp$user_old = df_full_temp$user
    df_full_temp$user = df_full_temp$temp.id
    
    # ecma    
    m_ecma_2b = lmer(scaled_score~-1+Xscore*task-Xscore-task+(1|user),df_full_temp)
    sigma_null = as.numeric(sqrt(VarCorr(m_ecma_2b)$user[1]))
    return(sigma_null)
  }
  
  # Save null distribution from ecma
  if(gen_new_null==T){
    ecma_null = replicate(n = 5000,draw_null_ecma())
    title_temp = "null_ecma MODEL 2b"
    write.csv(x = ecma_null,file = paste0(ifp,"/D_data/Nulls/",title_temp,".csv"),row.names = F)
  }
  
  if(gen_new_null==F){
    ecma_null = read.csv(paste0(ifp,"/D_data/Nulls/null_ecma MODEL 2b.csv"))
    ecma_null = ecma_null$x
  }
  
  # ECMA suggestion (col 3 of Table A1)  
  m_ecma_2b = lmer(scaled_score~-1+Xscore*task-Xscore-task+(1|user),df_full)
  summary(m_ecma_2b)
  confint(m_ecma_2b,level = 0.99)  
  
  # RANDOMIZATION INFERENCE for ecma
  VarCorr(m_ecma_2b)
  t_obs_ecma = 0.13932 # hard coded
  ecma_pval = mean(t_obs_ecma<ecma_null)
  paste0("pval from ecma = ",round(ecma_pval,3))
  hist(ecma_null)
  abline(v=t_obs_ecma)

  # Beta_ecma: fixed effects
  # Set up indicator matrix for ECMA model [i.e. fixed effects]
  imatrix = data.frame(group = final_groups,stringsAsFactors = F)
  
  # Add a column for each user
  for(i in 1:length(final_participants)) imatrix[,i+1] = 0
  names(imatrix)[-1] = paste0("user_",final_participants)
  
  # populate the matrix
  for(g in 1:nrow(imatrix)){
    g.temp = imatrix$group[g]
    ids.temp = allocation_final %>% filter(group == g.temp) %>% pull(user)
    ids.temp = paste0("user_",ids.temp)
    imatrix[g,which(colnames(imatrix) %in% ids.temp)] <- 1
  }
  
  # ecma regression (fixed effects)
  # Set up data
  df_ecma = left_join(df_all_expanded %>% dplyr::select(group,task,scaled_score,Xscore,prev,know),
                       imatrix,"group")
  
  # make sure our predictors are scaled
  # run regression
  mod_ecma = paste0("scaled_score~-1+Xscore*task-Xscore-task+know+prev+",paste0(names(imatrix)[-1],collapse = "+"))
  m_ecma_1b = lm(as.formula(mod_ecma),df_ecma)
  # grab fixed effect betas
  coef_ecma = summary(m_ecma_1b)$coef
  ecmabeta = coef_ecma[grep("user",rownames(coef_ecma)),1]
  beta_ecma = data.frame(user=gsub("user_","",names(ecmabeta),fixed=T),stringsAsFactors = F)
  beta_ecma$beta_ecma = ecmabeta

# Figure A8 -------------------------------------
  # Function to generate plot
  dbeta2 = as.data.frame(dbeta)
  dbeta2 = left_join(dbeta,beta_ecma,"user")
  plot_beta_2 = function(xvar,col,xlabel,df=dbeta2,save=FALSE,size=12){
    df = as.data.frame(df)
    df$x = scalex(df[,xvar])
    pb = ggplot(data=df,aes(x = x,y=beta_ecma))+
      geom_point(col=col,alpha=0.2,size=2)+
      theme_bw()+ggtitle("")+theme(text = element_text(size=16))+
      xlab(xlabel)+ylab("")+
      geom_smooth(method="lm",col=col)
    print(pb)
    return(pb)
  }
  
  # Beta and Ravens (Figure A8, left panel)
  pb_ravens = plot_beta_2(xvar="shapes",col="red",xlabel="Ravens Score [Scaled]")
  cor.test(dbeta2$shapes,dbeta2$beta_ecma)
  
  # Beta and RMET (Figure A8, right panel)
  pb_rmet = plot_beta_2(xvar="RMET",col="blue",xlabel="RMET Score [Scaled]")
  cor.test(dbeta2$RMET,dbeta2$beta_ecma)
  
  
# Table A7: Is the reporter associated with the teamplayer index ----------------
  # First, how stable is the reporter role?
  drep = drep %>% ungroup() %>% group_by(user) %>% summarize(n=n(),reporter_count=sum(reporter==TRUE))
  table(drep$reporter_count)

  # Add teamplayer index onto this count
  drep$user = as.character(drep$user)
  drep = left_join(drep,dbeta[,c("user","beta_fixef")],by="user")
  cor.test(drep$reporter_count,drep$beta_fixef)
  tapply(drep$beta_fixef,drep$reporter_count,mean)
  
  # Extra checks, where we look at different reporter status
    # add categorical variable defining reporter status
    drep$reporter_status = NA
    drep$reporter_status[drep$reporter_count %in% c(1,2,3)] = "Sometimes_Reporter"
    drep$reporter_status[drep$reporter_count==drep$n] = "Always_Reporter"
    drep$reporter_status[drep$reporter_count %in% c(0)] = "Never Reporter"
    mrep1 = lm(beta_fixef~as.factor(reporter_status)-1,drep); summary(mrep1)
    "no association here"
  
# Table A5: Is the team player index sensitive to task type ---------------------
task_mod = "scaled_score~Xscore" # pre-registered model, applied to one task

# Memory----
cov_temp = "task_analysis_memory" # name of model1
m1_temp = task_mod  # definition of model1
title_temp = paste("null",cov_temp,sep="_") # summary name of null distribution
if(gen_new_null==T) assign(title_temp,gen_save_null(df = subset(df_all_expanded,task=="memory"))) # if we want to generate a new null, do it here
if(gen_new_null==F) assign(title_temp,read.csv(paste0(ifp,"/D_data/Nulls/",title_temp,".csv"))) # otherwise, grab old null
memory_only_analysis = full_analysis(m1_form = m1_temp, 
                                     package="lmer",
                                     inference = "randomization",
                                     null_dist = title_temp,
                                     df = subset(df_all_expanded,task=="memory"))

# Shapes----
cov_temp = "task_analysis_shapes" # name of model1
m1_temp = task_mod  # definition of model1
title_temp = paste("null",cov_temp,sep="_") # summary name of null distribution
if(gen_new_null==T) assign(title_temp,gen_save_null(df = subset(df_all_expanded,task=="shapes"))) # if we want to generate a new null, do it here
if(gen_new_null==F) assign(title_temp,read.csv(paste0(ifp,"/D_data/Nulls/",title_temp,".csv"))) # otherwise, grab old null
shapes_only_analysis = full_analysis(m1_form = m1_temp, 
                                     package="lmer",
                                     inference = "randomization",
                                     null_dist = title_temp,
                                     df = subset(df_all_expanded,task=="shapes"))

# Optimization----
cov_temp = "task_analysis_opt" # name of model1
m1_temp = task_mod  # definition of model1
title_temp = paste("null",cov_temp,sep="_") # summary name of null distribution
if(gen_new_null==T) assign(title_temp,gen_save_null(df = subset(df_all_expanded,task=="opt"))) # if we want to generate a new null, do it here
if(gen_new_null==F) assign(title_temp,read.csv(paste0(ifp,"/D_data/Nulls/",title_temp,".csv"))) # otherwise, grab old null
opt_only_analysis = full_analysis(m1_form = m1_temp, 
                                     package="lmer",
                                     inference = "randomization",
                                     null_dist = title_temp,
                                     df = subset(df_all_expanded,task=="opt"))

# Correlations between beta_fixef and rmet [for column 2]
beta_cor_test = function(beta_out){
  df = left_join(beta_out,dbeta[,c("user","RMET")],"user")
  print(cor.test(df$beta_fixef,df$RMET))
}
beta_cor_test(memory_only_analysis$betas) # memory
beta_cor_test(shapes_only_analysis$betas) # shapes
beta_cor_test(opt_only_analysis$betas) # opt

# Correlations between sum_individual and group scores [for column 3]
with(subset(df_all_expanded,task=="memory"),cor.test(Xscore,scaled_score))
with(subset(df_all_expanded,task=="shapes"),cor.test(Xscore,scaled_score))
with(subset(df_all_expanded,task=="opt"),cor.test(Xscore,scaled_score))


# Table A6: which teams that are more than the sum of their parts ---------------
# Reporting 3.6 Characteristics of teams who outperform expectations --------
# Note: an extension here would be to do it at the task-group level, rather than the group level
# Get summary of group characteristics
Gx = t2_profile$Tg

# For each group, we need to add 8 variables
# Xscore_g: the overall level of individual skill in group g 
# Max_g: the individual skill level of the strongest member of group g (max(X_i) in group g)
# RMET_g: sum of individual RMET in group g
# DIV_g = diversity of each group, with respect to age; gender; individual skill. See definition, below.
# DivX: diversity of skill in group g (Max_g-Min_g)
# Div_Eth: binary indicator = 1 if all group members identify as the same race
# Div_fem: binary indicator = 1 if all group members identify as the gender
# Div_EF: binary indicator = 1 if all group members are the same gender and race

# Setup all our variables
G_in = df_all_expanded %>% group_by(group) %>%
  summarize(Xscore=Xscore[1],Xmax=Xmax[1],Xmin=Xmin[1],DivX=Xmax-Xmin,
            RMETsum=RMETsum[1],RMETmax=RMETmax[1],RMETmin=RMETmin[1],DivRMET=RMETmax-RMETsum)
Gx = left_join(Gx,G_in,"group")

# set up other columns
Gx$Divg = Gx$Div_ethn = Gx$Div_fem = Gx$Div_EF = Gx$count = NA

# Get inputs ready for mahalanobis
ifinal = df_full %>% group_by(user) %>% 
  summarize(Xscore=Xscore[1],
            memory=memory_scaled_i[1],
            opt_ave=opt_scaled_i[1],
            shapes=shapes_scaled_i[1],
            female=female_i[1],
            age=age_i[1],
            eduNUM=post_secondary_years_i[1],
            ethnicity=under_rep_minority_i[1])
ifinal = ifinal %>% filter(user %in% allocation$user)
mahal = ifinal
mahal = mahal %>% filter(user %in% final_participants)
mahal = mahal[complete.cases(mahal),]
mahal = as.matrix(mahal[,c("Xscore","age","female","eduNUM")])
mahalCOV = cov(mahal)

# Go and get information
for(g in 1:nrow(Gx)){
  gtemp = Gx$group[g]
  ids = sort(allocation$user[allocation$group %in% gtemp])
  
  # demographic info
  demotemp = ifinal %>% filter(user %in% ids) %>% dplyr::select(user,Xscore,age,eduNUM,female)
  
  # mini mahal
  mm = ifinal %>% filter(user %in% ids) %>% 
    dplyr::select(Xscore,age,female,eduNUM)
  mm = mm[complete.cases(mm),]
  mm = as.matrix(mm)
  
  # now we can get the mahalanobis for each pair
  n = nrow(mm)
  if(is.null(n)) out = NA
  if(!is.null(n)){
    if(n==1) out = NA
    if(n==2) out = mahalanobis(x = mm[1,],center = mm[2,],cov = mahalCOV,inverted = FALSE)
  }
  if(!is.null(n)){
    if(n==3){
      d12 = mahalanobis(x = mm[1,],center = mm[2,],cov = mahalCOV,inverted = FALSE)
      d13 = mahalanobis(x = mm[1,],center = mm[3,],cov = mahalCOV,inverted = FALSE)
      d23 = mahalanobis(x = mm[2,],center = mm[3,],cov = mahalCOV,inverted = FALSE)
      out = mean(d12,d13,d23)
    }}
  
  Gx$Divg[g] = out
  Gx$count[g] = ifelse(is.null(n),1,n)
  
  # other diversity measures
  # useful function
  all.same = function(vec){
    first = vec[1]
    all(vec==first)
  }
  # summarize remaining variables
  Gx$Div_fem[g] = all.same(ifinal %>% filter(user %in% ids) %>% pull(female))
  Gx$Div_ethn[g] = all.same(ifinal %>% filter(user %in% ids) %>% pull(ethnicity))
}

# Add final column for interaction between Div_fem and Div_eth
Gx$Div_EF = Gx$Div_ethn*Gx$Div_fem

# Analyse the qualities of an over-performing team (Table A6)
cor.test(Gx$Xscore,Gx$Tg)
cor.test(Gx$Xmax,Gx$Tg)
cor.test(Gx$RMETsum,Gx$Tg)
cor.test(Gx$Divg,Gx$Tg)
cor.test(Gx$DivX,Gx$Tg)
cor.test(as.numeric(Gx$Div_ethn),Gx$Tg)
cor.test(as.numeric(Gx$Div_fem),Gx$Tg)
cor.test(as.numeric(Gx$Div_EF),Gx$Tg)

# For text
cor.test(as.numeric(Gx$RMETmin),Gx$Tg)

# Figure A7: Is beta normally distributed ----------------------------------------
# Take beta fixed effects (from ecma model suggestion)
plot(density(dbeta$beta_fixef),xlim=c(-4,4))
curve(dnorm(x,mean=mean(dbeta$beta_fixef)+0.08,sd = 1.08),add=T,col="blue")
# "Note: in previous analyses, we used the sequential approach to get beta_fixef
# In response to the editor's comments, we use model1b, i.e. the simultaneous".


# Table A3: are we using the best individual predictors? -------------
df_all_expanded = df_all_expanded %>% group_by(group) %>% 
  mutate(Xmemory = Xscore[task=="memory"],
         Xopt = Xscore[task=="opt"],
         Xshapes = Xscore[task=="shapes"])

# Memory
memmod1 = lm(scaled_score~Xmemory,filter(df_all_expanded,task=="memory"))
memmod2 = lm(scaled_score~Xopt,filter(df_all_expanded,task=="memory"))
memmod3 = lm(scaled_score~Xshapes,filter(df_all_expanded,task=="memory"))
memmod4 = lm(scaled_score~Xopt+Xshapes,filter(df_all_expanded,task=="memory"))

stargazer(memmod1,memmod2,memmod3,memmod4,
          out=paste0(ifp,"/E_regression output/TABLE A3_Predictor Table_MEMORY.html"),
          column.sep.width = "-15pt",
          dep.var.labels = "Gmemory")

# Optimization
optmod1 = lm(scaled_score~Xopt,filter(df_all_expanded,task=="opt"))
optmod2 = lm(scaled_score~Xmemory,filter(df_all_expanded,task=="opt"))
optmod3 = lm(scaled_score~Xshapes,filter(df_all_expanded,task=="opt"))
optmod4 = lm(scaled_score~Xmemory+Xshapes,filter(df_all_expanded,task=="opt"))

stargazer(optmod1,optmod2,optmod3,optmod4,
          out=paste0(ifp,"/E_regression output/TABLE A3_Predictor Table_OPT.html"),
          column.sep.width = "-15pt",
          dep.var.labels = "Gopt")

# Shapes
shamod1 = lm(scaled_score~Xshapes,filter(df_all_expanded,task=="shapes"))
shamod2 = lm(scaled_score~Xmemory,filter(df_all_expanded,task=="shapes"))
shamod3 = lm(scaled_score~Xopt,filter(df_all_expanded,task=="shapes"))
shamod4 = lm(scaled_score~Xmemory+Xopt,filter(df_all_expanded,task=="shapes"))

stargazer(shamod1,shamod2,shamod3,shamod4,
          out=paste0(ifp,"/E_regression output/TABLE A3_Predictor Table_SHAPES.html"),
          column.sep.width = "-15pt",
          dep.var.labels = "Gopt")

# Table A4: model 1c ----------------------------------------------------------
# Measurement Simulation 
  # We do two things here: 
    # i) change the model as per the editor's suggestion (to include all X's); 
    # ii) add noise to each X

# Define suggested model
m_ecma_measure = "scaled_score~Xmemory+Xshapes+Xopt"

# Recall that the standard deviation of the Xik is 1 already
# Prepare a table with individual X values
# correlation between beta and X
Xind = df_full %>% 
  group_by(user) %>% 
  summarize(mem = memory_scaled_i[1],
            sha = shapes_scaled_i[1],
            opt = opt_scaled_i[1])
Xind$Xave = 1/3*(Xind$mem+Xind$sha+Xind$opt)

# These scaled_i variables are the ones that make up Xscore (e.g. in df_full)
add_X_noise = function(c){
  # take a level of noise c\sigma, and return a noisy version of df_all_expanded
  df_out = df_all_expanded %>% dplyr::select(group,task,scaled_score,Xscore)
  
  # start off with original X scores; add noise and rescale
  Xtemp = Xind %>% dplyr::select(user,mem,sha,opt)
  Xtemp$mem = rnorm(nrow(Xtemp),Xtemp$mem,sd = c)
  Xtemp$sha = rnorm(nrow(Xtemp),Xtemp$sha,sd = c)
  Xtemp$opt = rnorm(nrow(Xtemp),Xtemp$opt,sd = c)
  
  # Scale
  Xtemp$mem = scalex(Xtemp$mem)
  Xtemp$sha = scalex(Xtemp$sha)
  Xtemp$opt = scalex(Xtemp$opt)
  
  # link groups to users
  Xlong = df_full %>% dplyr::select(group,user,task)
  Xlong = left_join(Xlong,Xtemp,"user")
  Xlong_group = Xlong %>% group_by(group,task) %>% 
    summarize(Xmemory = mean(mem,na.rm=T)*3,
              Xshapes = mean(sha,na.rm=T)*3,
              Xopt = mean(opt,na.rm=T)*3)
  Xlong_group = Xlong_group %>% mutate(Xscore_star = case_when(
    task=="opt" ~ Xopt,
    task=="memory" ~ Xmemory,
    task=="shapes" ~ Xshapes)) %>% 
    dplyr::select(group,task,Xscore_star,Xmemory,Xshapes,Xopt)
  
  # put this back into df_out
  df_out = left_join(df_out,Xlong_group,c("group","task"))
  return(df_out)
}

# Analyse the newly-noisy data (with the suggested model from econometrica)
measurement_analysis = function(df,m1_form = m_ecma_measure){
  m1 = lm(as.formula(m1_form),df) # Step 1: fit model1
  df$res = m1$residuals # add residuals into data
  Tg = df %>% group_by(group) %>% summarize(Tg = mean(res)) # Step 2: get averages for each group
  
  # In-between step: attach individual IDs to groups [and subset to remove people whose data we don't analyse]
  Tg_i = left_join(subset(allocation_final,group %in% Tg$group),Tg,by="group") # add individuals
  Tg_i = Tg_i %>% filter(user %in% final_participants) # Just include participants who will be analysed
  
  # Step 4 Fit multilevel model
  mlm = lmer(Tg~1|user,Tg_i,REML=T)
  t_obs = sqrt(VarCorr(mlm)$user[1])
  
  # Get p-value from RI
  ri_null_temp = draw_null(m1_form = m_ecma_measure,
                           nsims=500, # for every run through, we need to generate a null distribution; keeping this to 500 so it doesn't take forever
                           df_inf=df)
  sigma_beta_pval_RI = mean(ri_null_temp>t_obs)
  
  # Calculate correlation with RMET
  beta_fixef = Tg_i %>% group_by(user) %>% 
    summarize(beta_fixef = mean(Tg))
  
  rho_dat_temp = left_join(dbeta %>% dplyr::select(user,eyes_sum),beta_fixef,"user")
  cor_temp = cor.test(rho_dat_temp$eyes_sum,rho_dat_temp$beta_fixef)
    
  # get output
  out = data.frame(rho_rmet = cor_temp$estimate, 
                   rho_rmet_pval = cor_temp$p.value,
                   sigma_beta = t_obs,
                   sigma_beta_pval_RI = sigma_beta_pval_RI)
  return(out)
}

# Run the simulation
run_sim = function(nsim=1,c){
  out_df = matrix(ncol=4,nrow=nsim)
  for(i in 1:nsim){
    new_dat = add_X_noise(c)
    out_list = measurement_analysis(new_dat)
    out_df[i,] = unlist(out_list)
  }
  out_df = as.data.frame(out_df)
  names(out_df) = c("rho_rmet","rho_rmet_p","sigma_beta","sigma_beta_p")
  return(out_df)
}

# Generate numbers for Table A4
c0 = run_sim(nsim=1,c=0) # warning: this takes a bit of time to run (around a minute)
nsims = 500 # number of times we want to run the simulations

if(gen_new_null==T){
c025 = run_sim(nsim=nsims,c=0.25)
write.csv(c025,paste0(ifp,"/D_data/Measure simulation/measure_simulation_c025.csv"),row.names = F)

c050 = run_sim(nsim=nsims,c=0.50)
write.csv(c050,paste0(ifp,"/D_data/Measure simulation/measure_simulation_c050.csv"),row.names = F)

c075 = run_sim(nsim=nsims,c=0.75)
write.csv(c075,paste0(ifp,"/D_data/Measure simulation/measure_simulation_c075.csv"),row.names = F)

} else {
  c025 = read_csv(paste0(ifp,"/D_data/Measure simulation/measure_simulation_c025.csv"))
  c050 = read_csv(paste0(ifp,"/D_data/Measure simulation/measure_simulation_c050.csv"))
  c075 = read_csv(paste0(ifp,"/D_data/Measure simulation/measure_simulation_c075.csv"))
}

# Summarize results for Table A4
apply(c0,2,mean)
apply(c025,2,mean)
apply(c050,2,mean)
apply(c075,2,mean)

